@@ -3,7 +3,7 @@
 //   For website terms of use, trademark policy, privacy policy and other
 //   project policies see https://lfprojects.org/policies
 // Copyright (c) 2020, Control Command Inc. All rights reserved.
-// Copyright (c) 2018-2021, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -27,6 +27,7 @@ import (
 
 	"github.com/apptainer/apptainer/docs"
 	"github.com/apptainer/apptainer/internal/pkg/buildcfg"
+	"github.com/apptainer/apptainer/internal/pkg/ociplatform"
 	"github.com/apptainer/apptainer/internal/pkg/plugin"
 	"github.com/apptainer/apptainer/internal/pkg/remote"
 	"github.com/apptainer/apptainer/internal/pkg/remote/endpoint"
@@ -40,7 +41,8 @@ import (
 	"github.com/apptainer/apptainer/pkg/util/apptainerconf"
 	keyClient "github.com/apptainer/container-key-client/client"
 	libClient "github.com/apptainer/container-library-client/client"
-	ocitypes "github.com/containers/image/v5/types"
+	"github.com/google/go-containerregistry/pkg/authn"
+	ggcrv1 "github.com/google/go-containerregistry/pkg/v1"
 	"github.com/spf13/cobra"
 	"golang.org/x/term"
 )
@@ -56,9 +58,9 @@ var CurrentUser = getCurrentUser()
 var currentRemoteEndpoint *endpoint.Config
 
 var (
-	dockerAuthConfig ocitypes.DockerAuthConfig
-	dockerLogin      bool
-	dockerHost       string
+	authConfig  authn.AuthConfig
+	dockerLogin bool
+	dockerHost  string
 
 	encryptionPEMPath   string
 	promptForPassphrase bool
@@ -66,6 +68,11 @@ var (
 	noHTTPS             bool
 	useBuildConfig      bool
 	tmpDir              string
+	// Optional user requested authentication file for writing/reading OCI registry credentials
+	reqAuthFile string
+	// Platform for retrieving images
+	arch     string
+	platform string
 )
 
 // apptainer command flags
@@ -136,7 +143,7 @@ var singVerboseFlag = cmdline.Flag{
 // --docker-username
 var dockerUsernameFlag = cmdline.Flag{
 	ID:            "dockerUsernameFlag",
-	Value:         &dockerAuthConfig.Username,
+	Value:         &authConfig.Username,
 	DefaultValue:  "",
 	Name:          "docker-username",
 	Usage:         "specify a username for docker authentication",
@@ -148,7 +155,7 @@ var dockerUsernameFlag = cmdline.Flag{
 // --docker-password
 var dockerPasswordFlag = cmdline.Flag{
 	ID:            "dockerPasswordFlag",
-	Value:         &dockerAuthConfig.Password,
+	Value:         &authConfig.Password,
 	DefaultValue:  "",
 	Name:          "docker-password",
 	Usage:         "specify a password for docker authentication",
@@ -258,6 +265,16 @@ var singBuildConfigFlag = cmdline.Flag{
 	Usage:        "use configuration needed for building containers",
 }
 
+// --authfile
+var commonAuthFileFlag = cmdline.Flag{
+	ID:           "commonAuthFileFlag",
+	Value:        &reqAuthFile,
+	DefaultValue: "",
+	Name:         "authfile",
+	Usage:        "Docker-style authentication file to use for writing/reading OCI registry credentials",
+	EnvKeys:      []string{"AUTH_FILE"},
+}
+
 func getCurrentUser() *user.User {
 	usr, err := user.Current()
 	if err != nil {
@@ -373,7 +390,7 @@ func handleConfDir(confDir, legacyConfigDir string) {
 	migrateKeys(confDir, legacyConfigDir)
 }
 
-func migrateRemoteConf(confDir, legacyConfigDir string) {
+func migrateRemoteConf(_, _ string) {
 	ok, err := fs.PathExists(syfs.LegacyRemoteConf())
 	if err != nil {
 		sylog.Warningf("Failed to retrieve information for %s: %s", syfs.LegacyRemoteConf(), err)
@@ -400,7 +417,7 @@ func migrateRemoteConf(confDir, legacyConfigDir string) {
 	}
 }
 
-func migrateDockerConf(confDir, legacyConfigDir string) {
+func migrateDockerConf(_, _ string) {
 	ok, err := fs.PathExists(syfs.LegacyDockerConf())
 	if err != nil {
 		sylog.Warningf("Failed to retrieve information for %s: %s", syfs.LegacyDockerConf(), err)
@@ -418,7 +435,7 @@ func migrateDockerConf(confDir, legacyConfigDir string) {
 	}
 }
 
-func migrateKeys(confDir, legacyConfigDir string) {
+func migrateKeys(_, _ string) {
 	legacySypgpDir := filepath.Join(syfs.LegacyConfigDir(), sypgp.LegacyDirectory)
 	keysDir := filepath.Join(syfs.ConfigDir(), sypgp.Directory)
 	ok, err := fs.PathExists(legacySypgpDir)
@@ -481,7 +498,7 @@ func migrateGPGPrivate(sypgpDir, legacySypgpDir string) {
 	}
 }
 
-func persistentPreRun(cmd *cobra.Command, args []string) error {
+func persistentPreRun(cmd *cobra.Command, _ []string) error {
 	setSylogMessageLevel()
 	sylog.Debugf("Apptainer version: %s", buildcfg.PACKAGE_VERSION)
 
@@ -617,7 +634,7 @@ func Init(loadPlugins bool) {
 var apptainerCmd = &cobra.Command{
 	TraverseChildren:      true,
 	DisableFlagsInUseLine: true,
-	RunE: func(cmd *cobra.Command, args []string) error {
+	RunE: func(_ *cobra.Command, _ []string) error {
 		return cmdline.CommandError("invalid command")
 	},
 
@@ -715,7 +732,7 @@ func TraverseParentsUses(cmd *cobra.Command) string {
 // VersionCmd displays installed apptainer version
 var VersionCmd = &cobra.Command{
 	DisableFlagsInUseLine: true,
-	Run: func(cmd *cobra.Command, args []string) {
+	Run: func(_ *cobra.Command, _ []string) {
 		fmt.Println(buildcfg.PACKAGE_VERSION)
 	},
 
@@ -814,7 +831,7 @@ func apptainerExec(image string, args []string) (string, error) {
 }
 
 // CheckRoot ensures that a command is executed with root privileges.
-func CheckRoot(cmd *cobra.Command, args []string) {
+func CheckRoot(cmd *cobra.Command, _ []string) {
 	if os.Geteuid() != 0 {
 		sylog.Fatalf("%q command requires root privileges", cmd.CommandPath())
 	}
@@ -822,7 +839,7 @@ func CheckRoot(cmd *cobra.Command, args []string) {
 
 // CheckRootOrUnpriv ensures that a command is executed with root
 // privileges or that Apptainer is installed unprivileged.
-func CheckRootOrUnpriv(cmd *cobra.Command, args []string) {
+func CheckRootOrUnpriv(cmd *cobra.Command, _ []string) {
 	if os.Geteuid() != 0 && buildcfg.APPTAINER_SUID_INSTALL == 1 {
 		sylog.Fatalf("%q command requires root privileges or an unprivileged installation", cmd.CommandPath())
 	}
@@ -880,3 +897,27 @@ func getLibraryClientConfig(uri string) (*libClient.Config, error) {
 	}
 	return libClientConfig, nil
 }
+
+// getOCIPlatform returns the appropriate OCI platform to use according to `--arch` and `--platform`
+func getOCIPlatform() ggcrv1.Platform {
+	var (
+		p   *ggcrv1.Platform
+		err error
+	)
+	if arch != "" && platform != "" {
+		err = fmt.Errorf("--arch and --platform cannot be used together")
+	}
+	if arch == "" && platform == "" {
+		p, err = ociplatform.DefaultPlatform()
+	}
+	if arch != "" {
+		p, err = ociplatform.PlatformFromArch(arch)
+	}
+	if platform != "" {
+		p, err = ociplatform.PlatformFromString(platform)
+	}
+	if err != nil {
+		sylog.Fatalf("%v", err)
+	}
+	return *p
+}
