@@ -21,7 +21,7 @@ import (
 //
 // apptainer plugin create <directory> <name>
 var PluginCreateCmd = &cobra.Command{
-	Run: func(cmd *cobra.Command, args []string) {
+	Run: func(_ *cobra.Command, args []string) {
 		name := args[1]
 		dir := args[0]
 
