@@ -2,7 +2,7 @@
 //   Apptainer a Series of LF Projects LLC.
 //   For website terms of use, trademark policy, privacy policy and other
 //   project policies see https://lfprojects.org/policies
-// Copyright (c) 2019-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2019-2024, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -469,7 +469,7 @@ func (c actionTests) STDPipe(t *testing.T) {
 			e2e.WithCommand(tt.command),
 			e2e.WithArgs(tt.argv...),
 			e2e.WithStdin(&input),
-			e2e.PreRun(func(t *testing.T) {
+			e2e.PreRun(func(_ *testing.T) {
 				input.WriteString(tt.input)
 			}),
 			e2e.ExpectExit(tt.exit),
@@ -698,52 +698,23 @@ func (c actionTests) RunFromURI(t *testing.T) {
 	}
 }
 
-// PersistentOverlay test the --overlay function
-func (c actionTests) PersistentOverlay(t *testing.T) {
-	e2e.EnsureImage(t, c.env)
-
-	require.Filesystem(t, "overlay")
-
-	require.Command(t, "mkfs.ext3")
-	require.Command(t, "mksquashfs")
-	require.Command(t, "dd")
-
-	testdir, err := os.MkdirTemp(c.env.TestDir, "persistent-overlay-")
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	cleanup := func(t *testing.T) {
-		if t.Failed() {
-			t.Logf("Not removing directory %s for test %s", testdir, t.Name())
-			return
-		}
-		err := os.RemoveAll(testdir)
-		if err != nil {
-			t.Logf("Error while removing directory %s for test %s: %#v", testdir, t.Name(), err)
-		}
-	}
-	// sandbox overlay implies creation of upper/work directories by
-	// Apptainer, so we would need privileges to delete the test
-	// directory correctly
-	defer e2e.Privileged(cleanup)
-
-	squashfsImage := filepath.Join(testdir, "squashfs.simg")
-	ext3Img := filepath.Join(testdir, "ext3_fs.img")
-	sandboxImage := filepath.Join(testdir, "sandbox")
-
-	// create an overlay directory
+func (c actionTests) overlayCreate(t *testing.T, testdir string) string {
+	// create overlay dir
 	dir, err := os.MkdirTemp(testdir, "overlay-dir-")
 	if err != nil {
 		t.Fatal(err)
 	}
+	return dir
+}
 
+func (c actionTests) squashfsCreate(t *testing.T, testdir string) string {
+	// squashfs img dir
+	squashfsImage := filepath.Join(testdir, "squashfs.simg")
 	// create root directory for squashfs image
 	squashDir, err := os.MkdirTemp(testdir, "root-squash-dir-")
 	if err != nil {
 		t.Fatal(err)
 	}
-
 	squashMarkerFile := "squash_marker"
 	if err := fs.Touch(filepath.Join(squashDir, squashMarkerFile)); err != nil {
 		t.Fatal(err)
@@ -754,24 +725,33 @@ func (c actionTests) PersistentOverlay(t *testing.T) {
 	if res := cmd.Run(t); res.Error != nil {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
+	return squashfsImage
+}
 
+func (c actionTests) ext3Create(t *testing.T, testdir string) string {
+	// create the ext3 overlay image
+	ext3Img := filepath.Join(testdir, "ext3_fs.img")
 	// create the overlay ext3 image
-	cmd = exec.Command("dd", "if=/dev/zero", "of="+ext3Img, "bs=1M", "count=64", "status=none")
+	cmd := exec.Command("dd", "if=/dev/zero", "of="+ext3Img, "bs=1M", "count=64", "status=none")
 	if res := cmd.Run(t); res.Error != nil {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
-
 	cmd = exec.Command("mkfs.ext3", "-q", "-F", ext3Img)
 	if res := cmd.Run(t); res.Error != nil {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
+	return ext3Img
+}
 
+func (c actionTests) sandboxCreate(t *testing.T, testdir string) string {
+	// create the sandbox dir
+	sandboxImage := filepath.Join(testdir, "sandbox")
 	// create a sandbox image from test image
 	c.env.RunApptainer(
 		t,
-		e2e.WithProfile(e2e.RootProfile),
+		e2e.WithProfile(e2e.UserProfile),
 		e2e.WithCommand("build"),
-		e2e.WithArgs("--sandbox", sandboxImage, c.env.ImagePath),
+		e2e.WithArgs("--force", "--sandbox", sandboxImage, c.env.ImagePath),
 		e2e.PostRun(func(t *testing.T) {
 			if t.Failed() {
 				t.Fatalf("failed to create sandbox %s from test image %s", sandboxImage, c.env.ImagePath)
@@ -779,131 +759,193 @@ func (c actionTests) PersistentOverlay(t *testing.T) {
 		}),
 		e2e.ExpectExit(0),
 	)
+	return sandboxImage
+}
+
+func (c actionTests) embeddedSifCreate(t *testing.T, testdir string) string {
+	// create overlay embedded img
+	embeddedSif := filepath.Join(testdir, "embedded.sif")
+	err := fs.CopyFile(c.env.ImagePath, embeddedSif, 0o700)
+	if err != nil {
+		t.Fatalf("Failed to copy file from %s to %s", c.env.ImagePath, embeddedSif)
+	}
+
+	c.env.RunApptainer(
+		t,
+		e2e.WithProfile(e2e.UserProfile),
+		e2e.WithCommand("overlay"),
+		e2e.WithArgs("create", embeddedSif),
+		e2e.ExpectExit(0),
+	)
+
+	return embeddedSif
+}
+
+func (c actionTests) PersistentOverlay(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	require.Filesystem(t, "overlay")
+
+	require.Command(t, "mkfs.ext3")
+	require.Command(t, "mksquashfs")
+	require.Command(t, "dd")
+
+	// the order should not be changed, it'll cause permission denied error. RootProfile will create
+	// overlay folder with the permission other users can not access.
+	profiles := []e2e.Profile{e2e.UserProfile, e2e.UserNamespaceProfile, e2e.RootProfile}
+
+	// create the test dir
+	testdir, err := os.MkdirTemp(c.env.TestDir, "persitent-overlay-")
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	defer e2e.Privileged(func(t *testing.T) {
+		if t.Failed() {
+			t.Logf("Not removing directory %s for test %s", testdir, t.Name())
+			return
+		}
+		err := os.RemoveAll(testdir)
+		if err != nil {
+			t.Logf("Error while removing directory %s for test %s: %#v", testdir, t.Name(), err)
+		}
+	})(t)
+
+	dir := c.overlayCreate(t, testdir)
+	squashfsImage := c.squashfsCreate(t, testdir)
+	ext3Img := c.ext3Create(t, testdir)
+	sandboxImage := c.sandboxCreate(t, testdir)
+	embeddedSif := c.embeddedSifCreate(t, testdir)
 
 	tests := []struct {
-		name    string
-		argv    []string
-		dir     string
-		exit    int
-		profile e2e.Profile
+		name     string
+		argv     []string
+		fakeroot bool
+		exit     int
+		dir      string
 	}{
 		{
-			name:    "overlay_create",
-			argv:    []string{"--overlay", dir, c.env.ImagePath, "touch", "/dir_overlay"},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name:     "overlay_create",
+			argv:     []string{"--overlay", dir, c.env.ImagePath, "touch", "/dir_overlay"},
+			fakeroot: true,
+			exit:     0,
 		},
 		{
-			name:    "overlay_ext3_create",
-			argv:    []string{"--overlay", ext3Img, c.env.ImagePath, "touch", "/ext3_overlay"},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name:     "overlay_ext3_create",
+			argv:     []string{"--overlay", ext3Img, c.env.ImagePath, "touch", "/ext3_overlay"},
+			fakeroot: true,
+			exit:     0,
 		},
 		{
-			name:    "overlay_multiple_create",
-			argv:    []string{"--overlay", ext3Img, "--overlay", squashfsImage + ":ro", c.env.ImagePath, "touch", "/multiple_overlay_fs"},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name:     "overlay_multiple_create",
+			argv:     []string{"--overlay", ext3Img, "--overlay", squashfsImage + ":ro", c.env.ImagePath, "touch", "/multiple_overlay_fs"},
+			fakeroot: true,
+			exit:     0,
 		},
 		{
-			name:    "overlay_find",
-			argv:    []string{"--overlay", dir, c.env.ImagePath, "test", "-f", "/dir_overlay"},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name:     "overlay_find",
+			argv:     []string{"--overlay", dir, c.env.ImagePath, "test", "-f", "/dir_overlay"},
+			fakeroot: true,
+			exit:     0,
 		},
 		{
-			name:    "overlay_find_with_writable_fail",
-			argv:    []string{"--overlay", dir, "--writable", c.env.ImagePath, "true"},
-			exit:    255,
-			profile: e2e.RootProfile,
+			name: "overlay_find_with_writable_fail",
+			argv: []string{"--overlay", dir, "--writable", c.env.ImagePath, "true"},
+			exit: 255,
 		},
 		{
-			name:    "overlay_find_with_writable_tmpfs",
-			argv:    []string{"--overlay", dir + ":ro", "--writable-tmpfs", c.env.ImagePath, "test", "-f", "/dir_overlay"},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name: "overlay_find_with_writable_tmpfs",
+			argv: []string{"--overlay", dir + ":ro", "--writable-tmpfs", c.env.ImagePath, "test", "-f", "/dir_overlay"},
+			exit: 0,
 		},
 		{
-			name:    "overlay_find_with_writable_tmpfs_fail",
-			argv:    []string{"--overlay", dir, "--writable-tmpfs", c.env.ImagePath, "true"},
-			exit:    255,
-			profile: e2e.RootProfile,
+			name: "overlay_find_with_writable_tmpfs_fail",
+			argv: []string{"--overlay", dir, "--writable-tmpfs", c.env.ImagePath, "true"},
+			exit: 255,
 		},
 		{
-			name:    "overlay_ext3_find",
-			argv:    []string{"--overlay", ext3Img, c.env.ImagePath, "test", "-f", "/ext3_overlay"},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name:     "overlay_ext3_find",
+			argv:     []string{"--overlay", ext3Img, c.env.ImagePath, "test", "-f", "/ext3_overlay"},
+			fakeroot: true,
+			exit:     0,
 		},
 		{
-			name:    "overlay_multiple_writable_fail",
-			argv:    []string{"--overlay", ext3Img, "--overlay", ext3Img, c.env.ImagePath, "true"},
-			exit:    255,
-			profile: e2e.RootProfile,
+			name: "overlay_multiple_writable_fail",
+			argv: []string{"--overlay", ext3Img, "--overlay", ext3Img, c.env.ImagePath, "true"},
+			exit: 255,
 		},
 		{
-			name:    "overlay_squashFS_find",
-			argv:    []string{"--overlay", squashfsImage + ":ro", c.env.ImagePath, "test", "-f", fmt.Sprintf("/%s", squashMarkerFile)},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name: "overlay_squashFS_find",
+			argv: []string{"--overlay", squashfsImage + ":ro", c.env.ImagePath, "test", "-f", "/squash_marker"},
+			exit: 0,
 		},
 		{
-			name:    "overlay_squashFS_find_without_ro",
-			argv:    []string{"--overlay", squashfsImage, c.env.ImagePath, "test", "-f", fmt.Sprintf("/%s", squashMarkerFile)},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name: "overlay_squashFS_find_without_ro",
+			argv: []string{"--overlay", squashfsImage, c.env.ImagePath, "test", "-f", "/squash_marker"},
+			exit: 0,
 		},
 		{
-			name:    "overlay_multiple_find_ext3",
-			argv:    []string{"--overlay", ext3Img, "--overlay", squashfsImage + ":ro", c.env.ImagePath, "test", "-f", "/multiple_overlay_fs"},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name:     "overlay_multiple_find_ext3",
+			argv:     []string{"--overlay", ext3Img, "--overlay", squashfsImage + ":ro", c.env.ImagePath, "test", "-f", "/multiple_overlay_fs"},
+			fakeroot: true,
+			exit:     0,
 		},
 		{
-			name:    "overlay_multiple_find_squashfs",
-			argv:    []string{"--overlay", ext3Img, "--overlay", squashfsImage + ":ro", c.env.ImagePath, "test", "-f", fmt.Sprintf("/%s", squashMarkerFile)},
-			exit:    0,
-			profile: e2e.RootProfile,
+			name:     "overlay_multiple_find_squashfs",
+			argv:     []string{"--overlay", ext3Img, "--overlay", squashfsImage + ":ro", c.env.ImagePath, "test", "-f", "/squash_marker"},
+			fakeroot: true,
+			exit:     0,
 		},
 		{
-			name:    "overlay_noroot",
-			argv:    []string{"--overlay", dir, c.env.ImagePath, "true"},
-			exit:    255,
-			profile: e2e.UserProfile,
+			name:     "overlay_noroot",
+			argv:     []string{"--overlay", dir, c.env.ImagePath, "true"},
+			fakeroot: true,
+			exit:     0,
 		},
 		{
-			name:    "overlay_noflag",
-			argv:    []string{c.env.ImagePath, "test", "-f", "/foo_overlay"},
-			exit:    1,
-			profile: e2e.RootProfile,
+			name: "overlay_noflag",
+			argv: []string{c.env.ImagePath, "test", "-f", "/foo_overlay"},
+			exit: 1,
 		},
 		{
 			// https://github.com/apptainer/singularity/issues/4329
-			name:    "SIF_writable_without_overlay_partition_issue_4329",
-			argv:    []string{"--writable", c.env.ImagePath, "true"},
-			exit:    255,
-			profile: e2e.RootProfile,
+			name: "SIF_writable_without_overlay_partition_issue_4329",
+			argv: []string{"--writable", c.env.ImagePath, "true"},
+			exit: 255,
 		},
 		{
 			// https://github.com/apptainer/singularity/issues/4270
-			name:    "overlay_dir_relative_path_issue_4270",
-			argv:    []string{"--overlay", filepath.Base(dir), sandboxImage, "test", "-f", "/dir_overlay"},
-			dir:     filepath.Dir(dir),
-			exit:    0,
-			profile: e2e.RootProfile,
+			name:     "overlay_dir_relative_path_issue_4270",
+			argv:     []string{"--overlay", filepath.Base(dir), sandboxImage, "test", "-f", "/dir_overlay"},
+			dir:      filepath.Dir(dir),
+			fakeroot: true,
+			exit:     0,
+		},
+		{
+			name: "Embedded overlay partition in SIF",
+			argv: []string{embeddedSif, "ps"},
+			exit: 0,
 		},
 	}
 
-	for _, tt := range tests {
-		c.env.RunApptainer(
-			t,
-			e2e.AsSubtest(tt.name),
-			e2e.WithProfile(tt.profile),
-			e2e.WithDir(tt.dir),
-			e2e.WithCommand("exec"),
-			e2e.WithArgs(tt.argv...),
-			e2e.ExpectExit(tt.exit),
-		)
+	for _, profile := range profiles {
+		for _, tt := range tests {
+			var args []string
+			if (profile.String() == "User" || profile.String() == "UserNamespace") && tt.fakeroot {
+				args = append(args, "--fakeroot")
+			}
+			args = append(args, tt.argv...)
+
+			c.env.RunApptainer(
+				t,
+				e2e.AsSubtest(tt.name+"_"+profile.String()),
+				e2e.WithProfile(profile),
+				e2e.WithDir(tt.dir),
+				e2e.WithCommand("exec"),
+				e2e.WithArgs(args...),
+				e2e.ExpectExit(tt.exit),
+			)
+		}
 	}
 }
 
@@ -998,73 +1040,6 @@ func (c actionTests) actionBasicProfiles(t *testing.T) {
 	}
 }
 
-// PersistentOverlayUnpriv tests the --overlay function in non-setuid root mode
-func (c actionTests) PersistentOverlayUnpriv(t *testing.T) {
-	e2e.EnsureImage(t, c.env)
-
-	require.Filesystem(t, "overlay")
-
-	testdir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "persistent-overlay-", "")
-	defer cleanup(t)
-
-	// create an overlay directory
-	dir, err := os.MkdirTemp(testdir, "overlay-dir-")
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	tests := []struct {
-		name    string
-		argv    []string
-		dir     string
-		exit    int
-		profile e2e.Profile
-	}{
-		{
-			name:    "overlay_create",
-			argv:    []string{"--overlay", dir, c.env.ImagePath, "touch", "/dir_overlay"},
-			exit:    0,
-			profile: e2e.UserNamespaceProfile,
-		},
-		{
-			name:    "overlay_find",
-			argv:    []string{"--overlay", dir, c.env.ImagePath, "test", "-f", "/dir_overlay"},
-			exit:    0,
-			profile: e2e.UserNamespaceProfile,
-		},
-		{
-			name:    "overlay_find_with_writable_fail",
-			argv:    []string{"--overlay", dir, "--writable", c.env.ImagePath, "true"},
-			exit:    255,
-			profile: e2e.UserNamespaceProfile,
-		},
-		{
-			name:    "overlay_find_with_writable_tmpfs",
-			argv:    []string{"--overlay", dir + ":ro", "--writable-tmpfs", c.env.ImagePath, "test", "-f", "/dir_overlay"},
-			exit:    0,
-			profile: e2e.UserNamespaceProfile,
-		},
-		{
-			name:    "overlay_find_with_writable_tmpfs_fail",
-			argv:    []string{"--overlay", dir, "--writable-tmpfs", c.env.ImagePath, "true"},
-			exit:    255,
-			profile: e2e.UserNamespaceProfile,
-		},
-	}
-
-	for _, tt := range tests {
-		c.env.RunApptainer(
-			t,
-			e2e.AsSubtest(tt.name),
-			e2e.WithProfile(tt.profile),
-			e2e.WithDir(tt.dir),
-			e2e.WithCommand("exec"),
-			e2e.WithArgs(tt.argv...),
-			e2e.ExpectExit(tt.exit),
-		)
-	}
-}
-
 func (c actionTests) actionNetwork(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
@@ -1126,13 +1101,94 @@ func (c actionTests) actionNetwork(t *testing.T) {
 	}
 }
 
+func (c actionTests) actionNetnsPath(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+	require.Command(t, "ip")
+
+	nsName := "apptainer-e2e"
+	nsPath := filepath.Join("/run", "netns", nsName)
+
+	netnsInode := uint64(0)
+
+	e2e.Privileged(func(t *testing.T) {
+		t.Log("Creating netns")
+		cmd := exec.Command("ip", "netns", "add", nsName)
+		if res := cmd.Run(t); res.Error != nil {
+			t.Fatalf("While creating network namespace: %s", res)
+		}
+		fi, err := os.Stat(nsPath)
+		if err != nil {
+			t.Fatal(err)
+		}
+		stat, ok := fi.Sys().(*syscall.Stat_t)
+		if !ok {
+			t.Fatal("Stat_t assertion error")
+		}
+		netnsInode = stat.Ino
+		t.Logf("Netns inode: %d", netnsInode)
+	})(t)
+
+	defer e2e.Privileged(func(t *testing.T) {
+		t.Log("Deleting netns")
+		cmd := exec.Command("ip", "netns", "delete", nsName)
+		if res := cmd.Run(t); res.Error != nil {
+			t.Fatalf("While deleting network namespace: %s", res)
+		}
+	})(t)
+
+	tests := []struct {
+		name         string
+		profile      e2e.Profile
+		netnsPath    string
+		expectExit   int
+		expectOutput string
+	}{
+		{
+			name:         "root",
+			profile:      e2e.RootProfile,
+			netnsPath:    nsPath,
+			expectExit:   0,
+			expectOutput: fmt.Sprintf("%d", netnsInode),
+		},
+		{
+			name:       "user",
+			profile:    e2e.UserProfile,
+			netnsPath:  nsPath,
+			expectExit: 255,
+		},
+		{
+			name:       "userns",
+			profile:    e2e.UserNamespaceProfile,
+			netnsPath:  nsPath,
+			expectExit: 255,
+		},
+		{
+			name:       "fakeroot",
+			profile:    e2e.FakerootProfile,
+			netnsPath:  nsPath,
+			expectExit: 255,
+		},
+	}
+
+	for _, tt := range tests {
+		c.env.RunApptainer(
+			t,
+			e2e.AsSubtest(tt.name),
+			e2e.WithProfile(tt.profile),
+			e2e.WithCommand("exec"),
+			e2e.WithArgs("--netns-path", tt.netnsPath, c.env.ImagePath, "stat", "-L", "-c", "%i", "/proc/self/ns/net"),
+			e2e.ExpectExit(tt.expectExit, e2e.ExpectOutput(e2e.ContainMatch, tt.expectOutput)),
+		)
+	}
+}
+
 //nolint:maintidx
 func (c actionTests) actionBinds(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
 	workspace, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "bind-workspace-", "")
 	sandbox, _ := e2e.MakeTempDir(t, workspace, "sandbox-", "")
-	defer e2e.Privileged(cleanup)
+	defer e2e.Privileged(cleanup)(t)
 
 	contCanaryDir := "/canary"
 	hostCanaryDir := filepath.Join(workspace, "canary")
@@ -1925,7 +1981,7 @@ func (c actionTests) actionLayerType(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
 	sandbox, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "action-layertype-sandbox", "")
-	defer e2e.Privileged(cleanup)
+	defer e2e.Privileged(cleanup)(t)
 
 	// convert test image to sandbox
 	c.env.RunApptainer(
@@ -2096,7 +2152,7 @@ func (c actionTests) fuseMount(t *testing.T) {
 	u := e2e.UserProfile.HostUser(t)
 
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "sshfs-", "")
-	defer e2e.Privileged(cleanup)
+	defer e2e.Privileged(cleanup)(t)
 
 	sshfsWrapper := filepath.Join(imageDir, "sshfs-wrapper")
 	rootPrivKey := filepath.Join(imageDir, "/etc/ssh/ssh_host_rsa_key")
@@ -2139,7 +2195,7 @@ func (c actionTests) fuseMount(t *testing.T) {
 			e2e.WithStdin(stdinReader),
 			e2e.WithCommand("run"),
 			e2e.WithArgs("--writable", "--no-home", imageDir),
-			e2e.PostRun(func(t *testing.T) {
+			e2e.PostRun(func(_ *testing.T) {
 				stdinReader.Close()
 				stdinWriter.Close()
 			}),
@@ -3004,7 +3060,7 @@ func (c actionTests) relWorkdirScratch(t *testing.T) {
 	testdir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "persistent-overlay-", "")
 	t.Cleanup(func() {
 		if !t.Failed() {
-			e2e.Privileged(cleanup)
+			e2e.Privileged(cleanup)(t)
 		}
 	})
 
@@ -3037,6 +3093,128 @@ func (c actionTests) relWorkdirScratch(t *testing.T) {
 	}
 }
 
+// actionAuth tests run/exec/shell flows that involve authenticated pulls from
+// OCI registries.
+func (c actionTests) actionAuth(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+	profiles := []e2e.Profile{
+		e2e.UserProfile,
+		e2e.RootProfile,
+	}
+
+	for _, p := range profiles {
+		t.Run(p.String(), func(t *testing.T) {
+			t.Run("default", func(t *testing.T) {
+				c.actionAuthTester(t, false, p)
+			})
+			t.Run("custom", func(t *testing.T) {
+				c.actionAuthTester(t, true, p)
+			})
+		})
+	}
+}
+
+func (c actionTests) actionAuthTester(t *testing.T, withCustomAuthFile bool, profile e2e.Profile) {
+	tmpdir, tmpdirCleanup := e2e.MakeTempDir(t, c.env.TestDir, "action-auth", "")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			tmpdirCleanup(t)
+		}
+	})
+
+	prevCwd, err := os.Getwd()
+	if err != nil {
+		t.Fatalf("could not get current working directory: %s", err)
+	}
+	defer os.Chdir(prevCwd)
+	if err = os.Chdir(tmpdir); err != nil {
+		t.Fatalf("could not change cwd to %q: %s", tmpdir, err)
+	}
+
+	localAuthFileName := ""
+	if withCustomAuthFile {
+		localAuthFileName = "./my_local_authfile"
+	}
+
+	authFileArgs := []string{}
+	if withCustomAuthFile {
+		authFileArgs = []string{"--authfile", localAuthFileName}
+	}
+
+	t.Cleanup(func() {
+		e2e.PrivateRepoLogout(t, c.env, profile, localAuthFileName)
+	})
+
+	orasCustomPushTarget := fmt.Sprintf("oras://%s/authfile-pushtest-oras-busybox:latest", c.env.TestRegistryPrivPath)
+
+	tests := []struct {
+		name          string
+		cmd           string
+		args          []string
+		whileLoggedIn bool
+		expectExit    int
+	}{
+		{
+			name:          "docker before auth",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", c.env.TestRegistryPrivImage, "true"},
+			whileLoggedIn: false,
+			expectExit:    255,
+		},
+		{
+			name:          "docker",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", c.env.TestRegistryPrivImage, "true"},
+			whileLoggedIn: true,
+			expectExit:    0,
+		},
+		{
+			name:          "noauth docker",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", c.env.TestRegistryPrivImage, "true"},
+			whileLoggedIn: false,
+			expectExit:    255,
+		},
+		{
+			name:          "oras push",
+			cmd:           "push",
+			args:          []string{c.env.ImagePath, orasCustomPushTarget},
+			whileLoggedIn: true,
+			expectExit:    0,
+		},
+		{
+			name:          "noauth oras",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", orasCustomPushTarget, "true"},
+			whileLoggedIn: false,
+			expectExit:    255,
+		},
+		{
+			name:          "oras",
+			cmd:           "exec",
+			args:          []string{"--disable-cache", "--no-https", orasCustomPushTarget, "true"},
+			whileLoggedIn: true,
+			expectExit:    0,
+		},
+	}
+
+	for _, tt := range tests {
+		if tt.whileLoggedIn {
+			e2e.PrivateRepoLogin(t, c.env, profile, localAuthFileName)
+		} else {
+			e2e.PrivateRepoLogout(t, c.env, profile, localAuthFileName)
+		}
+		c.env.RunApptainer(
+			t,
+			e2e.AsSubtest(tt.name),
+			e2e.WithProfile(profile),
+			e2e.WithCommand(tt.cmd),
+			e2e.WithArgs(append(authFileArgs, tt.args...)...),
+			e2e.ExpectExit(tt.expectExit),
+		)
+	}
+}
+
 // E2ETests is the main func to trigger the test suite
 func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	c := actionTests{
@@ -3046,49 +3224,50 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	np := testhelper.NoParallel
 
 	return testhelper.Tests{
-		"action URI":                   c.RunFromURI,              // action_URI
-		"singularity link":             c.singularityLink,         // singularity symlink
-		"exec":                         c.actionExec,              // apptainer exec
-		"exec under multiple profiles": c.actionExecMultiProfile,  // apptainer exec
-		"persistent overlay":           c.PersistentOverlay,       // Persistent Overlay
-		"persistent overlay unpriv":    c.PersistentOverlayUnpriv, // Persistent Overlay Unprivileged
-		"run":                          c.actionRun,               // apptainer run
-		"shell":                        c.actionShell,             // shell interaction
-		"STDPIPE":                      c.STDPipe,                 // stdin/stdout pipe
-		"action basic profiles":        c.actionBasicProfiles,     // run basic action under different profiles
-		"issue 4488":                   c.issue4488,               // https://github.com/apptainer/singularity/issues/4488
-		"issue 4587":                   c.issue4587,               // https://github.com/apptainer/singularity/issues/4587
-		"issue 4755":                   c.issue4755,               // https://github.com/apptainer/singularity/issues/4755
-		"issue 4768":                   c.issue4768,               // https://github.com/apptainer/singularity/issues/4768
-		"issue 4797":                   c.issue4797,               // https://github.com/apptainer/singularity/issues/4797
-		"issue 4823":                   c.issue4823,               // https://github.com/apptainer/singularity/issues/4823
-		"issue 4836":                   c.issue4836,               // https://github.com/apptainer/singularity/issues/4836
-		"issue 5211":                   c.issue5211,               // https://github.com/apptainer/singularity/issues/5211
-		"issue 5228":                   c.issue5228,               // https://github.com/apptainer/singularity/issues/5228
-		"issue 5271":                   c.issue5271,               // https://github.com/apptainer/singularity/issues/5271
-		"issue 5399":                   c.issue5399,               // https://github.com/apptainer/singularity/issues/5399
-		"issue 5455":                   c.issue5455,               // https://github.com/apptainer/singularity/issues/5455
-		"issue 5465":                   c.issue5465,               // https://github.com/apptainer/singularity/issues/5465
-		"issue 5599":                   c.issue5599,               // https://github.com/apptainer/singularity/issues/5599
-		"issue 5631":                   c.issue5631,               // https://github.com/apptainer/singularity/issues/5631
-		"issue 5690":                   c.issue5690,               // https://github.com/apptainer/singularity/issues/5690
-		"issue 6165":                   c.issue6165,               // https://github.com/apptainer/singularity/issues/6165
-		"issue 619":                    c.issue619,                // https://github.com/apptainer/apptainer/issues/619
-		"issue 1097":                   c.issue1097,               // https://github.com/apptainer/apptainer/issues/1097
-		"issue 1848":                   c.issue1848,               // https://github.com/apptainer/apptainer/issues/1848
-		"network":                      c.actionNetwork,           // test basic networking
-		"binds":                        c.actionBinds,             // test various binds with --bind and --mount
-		"layerType":                    c.actionLayerType,         // verify the various layer types
-		"exit and signals":             c.exitSignals,             // test exit and signals propagation
-		"fuse mount":                   c.fuseMount,               // test fusemount option
-		"bind image":                   c.bindImage,               // test bind image with --bind and --mount
-		"unsquash":                     c.actionUnsquash,          // test --unsquash
-		"no-mount":                     c.actionNoMount,           // test --no-mount
-		"compat":                       np(c.actionCompat),        // test --compat
-		"umask":                        np(c.actionUmask),         // test umask propagation
-		"invalidRemote":                np(c.invalidRemote),       // GHSA-5mv9-q7fq-9394
-		"fakeroot home":                c.actionFakerootHome,      // test home dir in fakeroot
-		"relWorkdirScratch":            np(c.relWorkdirScratch),   // test relative --workdir with --scratch
-		"issue 1868":                   c.issue1868,               // https://github.com/apptainer/apptainer/issues/1868
+		"action URI":                   c.RunFromURI,             // action_URI
+		"singularity link":             c.singularityLink,        // singularity symlink
+		"exec":                         c.actionExec,             // apptainer exec
+		"exec under multiple profiles": c.actionExecMultiProfile, // apptainer exec
+		"run":                          c.actionRun,              // apptainer run
+		"shell":                        c.actionShell,            // shell interaction
+		"STDPIPE":                      c.STDPipe,                // stdin/stdout pipe
+		"persistent overlay":           c.PersistentOverlay,
+		"action basic profiles":        c.actionBasicProfiles,   // run basic action under different profiles
+		"issue 4488":                   c.issue4488,             // https://github.com/apptainer/singularity/issues/4488
+		"issue 4587":                   c.issue4587,             // https://github.com/apptainer/singularity/issues/4587
+		"issue 4755":                   c.issue4755,             // https://github.com/apptainer/singularity/issues/4755
+		"issue 4768":                   c.issue4768,             // https://github.com/apptainer/singularity/issues/4768
+		"issue 4797":                   c.issue4797,             // https://github.com/apptainer/singularity/issues/4797
+		"issue 4823":                   c.issue4823,             // https://github.com/apptainer/singularity/issues/4823
+		"issue 4836":                   c.issue4836,             // https://github.com/apptainer/singularity/issues/4836
+		"issue 5211":                   c.issue5211,             // https://github.com/apptainer/singularity/issues/5211
+		"issue 5228":                   c.issue5228,             // https://github.com/apptainer/singularity/issues/5228
+		"issue 5271":                   c.issue5271,             // https://github.com/apptainer/singularity/issues/5271
+		"issue 5399":                   c.issue5399,             // https://github.com/apptainer/singularity/issues/5399
+		"issue 5455":                   c.issue5455,             // https://github.com/apptainer/singularity/issues/5455
+		"issue 5465":                   c.issue5465,             // https://github.com/apptainer/singularity/issues/5465
+		"issue 5599":                   c.issue5599,             // https://github.com/apptainer/singularity/issues/5599
+		"issue 5631":                   c.issue5631,             // https://github.com/apptainer/singularity/issues/5631
+		"issue 5690":                   c.issue5690,             // https://github.com/apptainer/singularity/issues/5690
+		"issue 6165":                   c.issue6165,             // https://github.com/apptainer/singularity/issues/6165
+		"issue 619":                    c.issue619,              // https://github.com/apptainer/apptainer/issues/619
+		"issue 1097":                   c.issue1097,             // https://github.com/apptainer/apptainer/issues/1097
+		"issue 1848":                   c.issue1848,             // https://github.com/apptainer/apptainer/issues/1848
+		"network":                      c.actionNetwork,         // test basic networking
+		"netns-path":                   c.actionNetnsPath,       // test netns joining
+		"binds":                        c.actionBinds,           // test various binds with --bind and --mount
+		"layerType":                    c.actionLayerType,       // verify the various layer types
+		"exit and signals":             c.exitSignals,           // test exit and signals propagation
+		"fuse mount":                   c.fuseMount,             // test fusemount option
+		"bind image":                   c.bindImage,             // test bind image with --bind and --mount
+		"unsquash":                     c.actionUnsquash,        // test --unsquash
+		"no-mount":                     c.actionNoMount,         // test --no-mount
+		"compat":                       np(c.actionCompat),      // test --compat
+		"umask":                        np(c.actionUmask),       // test umask propagation
+		"invalidRemote":                np(c.invalidRemote),     // GHSA-5mv9-q7fq-9394
+		"fakeroot home":                c.actionFakerootHome,    // test home dir in fakeroot
+		"relWorkdirScratch":            np(c.relWorkdirScratch), // test relative --workdir with --scratch
+		"issue 1868":                   c.issue1868,             // https://github.com/apptainer/apptainer/issues/1868
+		"auth":                         np(c.actionAuth),        // tests action cmds w/authenticated pulls from OCI registries
 	}
 }
