@@ -14,6 +14,7 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"syscall"
 	"testing"
 
 	"github.com/apptainer/apptainer/e2e/internal/e2e"
@@ -169,6 +170,24 @@ func (c configTests) configGlobal(t *testing.T) {
 			directiveValue: "0",
 			exit:           255,
 		},
+		{
+			name:           "AllowIpcNsNo",
+			argv:           []string{"--ipc", c.env.ImagePath, "readlink", "/proc/self/ns/ipc"},
+			profile:        e2e.UserProfile,
+			directive:      "allow ipc ns",
+			directiveValue: "no",
+			exit:           0,
+			resultOp:       e2e.ExpectOutput(e2e.ExactMatch, func() string { s, _ := os.Readlink("/proc/self/ns/ipc"); return s }()),
+		},
+		{
+			name:           "AllowIpcNsYes",
+			argv:           []string{"--ipc", c.env.ImagePath, "readlink", "/proc/self/ns/ipc"},
+			profile:        e2e.UserProfile,
+			directive:      "allow ipc ns",
+			directiveValue: "yes",
+			exit:           0,
+			resultOp:       e2e.ExpectOutput(e2e.UnwantedExactMatch, func() string { s, _ := os.Readlink("/proc/self/ns/ipc"); return s }()),
+		},
 		{
 			name:           "AllowPidNsNo",
 			argv:           []string{"--pid", "--no-init", c.env.ImagePath, "/bin/sh", "-c", "echo $$"},
@@ -187,6 +206,49 @@ func (c configTests) configGlobal(t *testing.T) {
 			exit:           0,
 			resultOp:       e2e.ExpectOutput(e2e.ExactMatch, "1"),
 		},
+		{
+			name:           "AllowUserNsNo",
+			argv:           []string{c.env.ImagePath, "true"},
+			profile:        e2e.UserNamespaceProfile,
+			directive:      "allow user ns",
+			directiveValue: "no",
+			exit:           255,
+		},
+		{
+			name:           "AllowUserNsNoFakeroot",
+			argv:           []string{c.env.ImagePath, "true"},
+			profile:        e2e.FakerootProfile,
+			directive:      "allow user ns",
+			directiveValue: "no",
+			exit:           255,
+		},
+		{
+			name:           "AllowUserNsYes",
+			argv:           []string{c.env.ImagePath, "readlink", "/proc/self/ns/user"},
+			profile:        e2e.UserNamespaceProfile,
+			directive:      "allow user ns",
+			directiveValue: "yes",
+			exit:           0,
+			resultOp:       e2e.ExpectOutput(e2e.UnwantedExactMatch, func() string { s, _ := os.Readlink("/proc/self/ns/user"); return s }()),
+		},
+		{
+			name:           "AllowUtsNsNo",
+			argv:           []string{"--uts", "--hostname", "foo", c.env.ImagePath, "hostname"},
+			profile:        e2e.UserProfile,
+			directive:      "allow uts ns",
+			directiveValue: "no",
+			exit:           0,
+			resultOp:       e2e.ExpectOutput(e2e.UnwantedExactMatch, "foo"),
+		},
+		{
+			name:           "AllowUtsNsYes",
+			argv:           []string{"--uts", "--hostname", "foo", c.env.ImagePath, "hostname"},
+			profile:        e2e.UserProfile,
+			directive:      "allow uts ns",
+			directiveValue: "yes",
+			exit:           0,
+			resultOp:       e2e.ExpectOutput(e2e.ExactMatch, "foo"),
+		},
 		{
 			name:           "ConfigPasswdNo",
 			argv:           []string{c.env.ImagePath, "grep", "/etc/passwd.*- tmpfs", "/proc/self/mountinfo"},
@@ -359,6 +421,7 @@ func (c configTests) configGlobal(t *testing.T) {
 			exit:           0,
 		},
 		// overlay may or not be available, just test with no
+		//nolint:dupword
 		{
 			name:           "EnableOverlayNo",
 			argv:           []string{c.env.ImagePath, "grep", "\\- overlay overlay", "/proc/self/mountinfo"},
@@ -716,19 +779,14 @@ func (c configTests) configGlobal(t *testing.T) {
 			directiveValue: "yes",
 			exit:           1,
 		},
-		// FIXME
-		// The e2e tests currently run inside a PID namespace.
-		//   (see internal/init/init_linux.go)
-		// We can't instruct systemd to manage our cgroups as the PIDs in our test namespace
-		// won't match what systemd sees.
-		// {
-		// 	name:           "SystemdCgroupsYes",
-		// 	argv:           []string{"--apply-cgroups", "testdata/cgroups/pids_limit.toml", c.sandboxImage, "true"},
-		// 	profile:        e2e.RootProfile,
-		// 	directive:      "systemd cgroups",
-		// 	directiveValue: "yes",
-		// 	exit:           0,
-		// },
+		{
+			name:           "SystemdCgroupsYes",
+			argv:           []string{"--apply-cgroups", "testdata/cgroups/pids_limit.toml", c.sandboxImage, "true"},
+			profile:        e2e.RootProfile,
+			directive:      "systemd cgroups",
+			directiveValue: "yes",
+			exit:           0,
+		},
 		{
 			name:           "SystemdCgroupNo",
 			argv:           []string{"--apply-cgroups", "testdata/cgroups/pids_limit.toml", c.sandboxImage, "true"},
@@ -991,6 +1049,127 @@ func (c configTests) configGlobalCombination(t *testing.T) {
 	}
 }
 
+func (c configTests) configUserNetns(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+	require.Command(t, "ip")
+
+	u := e2e.UserProfile.HostUser(t)
+	g, err := user.GetGrGID(u.GID)
+	if err != nil {
+		t.Fatalf("could not retrieve user group information: %s", err)
+	}
+
+	nsName := "singularity-e2e"
+	nsPath := filepath.Join("/run", "netns", nsName)
+
+	netnsInode := uint64(0)
+
+	e2e.Privileged(func(t *testing.T) {
+		t.Log("Creating netns")
+		cmd := exec.Command("ip", "netns", "add", nsName)
+		if err := cmd.Run(); err != nil {
+			t.Fatalf("While creating network namespace: %v", err)
+		}
+		fi, err := os.Stat(nsPath)
+		if err != nil {
+			t.Fatal(err)
+		}
+		stat, ok := fi.Sys().(*syscall.Stat_t)
+		if !ok {
+			t.Fatal("Stat_t assertion error")
+		}
+		netnsInode = stat.Ino
+		t.Logf("Netns inode: %d", netnsInode)
+	})(t)
+
+	defer e2e.Privileged(func(t *testing.T) {
+		t.Log("Deleting netns")
+		cmd := exec.Command("ip", "netns", "delete", nsName)
+		if err := cmd.Run(); err != nil {
+			t.Fatalf("While deleting network namespace: %v", err)
+		}
+	})(t)
+
+	tests := []struct {
+		name         string
+		netnsPath    string
+		directives   map[string]string
+		expectExit   int
+		expectOutput string
+	}{
+		{
+			name:       "Default",
+			netnsPath:  nsPath,
+			expectExit: 255,
+		},
+		{
+			name:      "NetnsPathOnly",
+			netnsPath: nsPath,
+			directives: map[string]string{
+				"allow netns paths": nsPath,
+			},
+			expectExit: 255,
+		},
+		{
+			name:      "NetUserOnly",
+			netnsPath: nsPath,
+			directives: map[string]string{
+				"allow net users": u.Name,
+			},
+			expectExit: 255,
+		},
+		{
+			name:      "NetGroupOnly",
+			netnsPath: nsPath,
+			directives: map[string]string{
+				"allow net groups": g.Name,
+			},
+			expectExit: 255,
+		},
+		{
+			name:      "ValidUser",
+			netnsPath: nsPath,
+			directives: map[string]string{
+				"allow net users":   u.Name,
+				"allow netns paths": nsPath,
+			},
+			expectExit:   0,
+			expectOutput: fmt.Sprintf("%d", netnsInode),
+		},
+		{
+			name:      "ValidGroup",
+			netnsPath: nsPath,
+			directives: map[string]string{
+				"allow net groups":  g.Name,
+				"allow netns paths": nsPath,
+			},
+			expectExit:   0,
+			expectOutput: fmt.Sprintf("%d", netnsInode),
+		},
+	}
+
+	for _, tt := range tests {
+		c.env.RunApptainer(
+			t,
+			e2e.AsSubtest(tt.name),
+			e2e.WithProfile(e2e.UserProfile),
+			e2e.PreRun(func(t *testing.T) {
+				for k, v := range tt.directives {
+					e2e.SetDirective(t, c.env, k, v)
+				}
+			}),
+			e2e.PostRun(func(t *testing.T) {
+				for k := range tt.directives {
+					e2e.ResetDirective(t, c.env, k)
+				}
+			}),
+			e2e.WithCommand("exec"),
+			e2e.WithArgs("--netns-path", tt.netnsPath, c.env.ImagePath, "stat", "-L", "-c", "%i", "/proc/self/ns/net"),
+			e2e.ExpectExit(tt.expectExit, e2e.ExpectOutput(e2e.ContainMatch, tt.expectOutput)),
+		)
+	}
+}
+
 func (c configTests) configFile(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
@@ -1063,5 +1242,6 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		"config file":               c.configFile,                  // test --config file option
 		"config global":             np(c.configGlobal),            // test various global configuration
 		"config global combination": np(c.configGlobalCombination), // test various global configuration with combination
+		"config user netns":         np(c.configUserNetns),         // test entering a network namespace as an unpriv user
 	}
 }
