@@ -186,7 +186,7 @@ func (c *imgBuildTests) issue4967(t *testing.T) {
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("build"),
 		e2e.WithArgs(image, "testdata/regressions/issue_4967.def"),
-		e2e.PostRun(func(t *testing.T) {
+		e2e.PostRun(func(_ *testing.T) {
 			os.Remove(image)
 		}),
 		e2e.ExpectExit(
@@ -206,7 +206,7 @@ func (c *imgBuildTests) issue4969(t *testing.T) {
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("build"),
 		e2e.WithArgs(image, "testdata/regressions/issue_4969.def"),
-		e2e.PostRun(func(t *testing.T) {
+		e2e.PostRun(func(_ *testing.T) {
 			os.Remove(image)
 		}),
 		e2e.ExpectExit(
@@ -276,7 +276,7 @@ func (c *imgBuildTests) issue4820(t *testing.T) {
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("build"),
 		e2e.WithArgs(image, "testdata/regressions/issue_4820.def"),
-		e2e.PostRun(func(t *testing.T) {
+		e2e.PostRun(func(_ *testing.T) {
 			os.Remove(image)
 		}),
 		e2e.ExpectExit(
@@ -295,7 +295,7 @@ func (c *imgBuildTests) issue5315(t *testing.T) {
 		e2e.WithProfile(e2e.FakerootProfile),
 		e2e.WithCommand("build"),
 		e2e.WithArgs(image, "testdata/regressions/issue_5315.def"),
-		e2e.PostRun(func(t *testing.T) {
+		e2e.PostRun(func(_ *testing.T) {
 			os.Remove(image)
 		}),
 		e2e.ExpectExit(
@@ -453,7 +453,7 @@ From: {{ .From }}
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("build"),
 		e2e.WithArgs(image, f.Name()),
-		e2e.PostRun(func(t *testing.T) {
+		e2e.PostRun(func(_ *testing.T) {
 			os.Remove(image)
 		}),
 		e2e.ExpectExit(0),
@@ -529,7 +529,7 @@ from: %s
 				t, c.env.TestDir, fmt.Sprintf("issue1812-sandbox-%s-", testName), "")
 			t.Cleanup(func() {
 				if !t.Failed() {
-					e2e.Privileged(cleanup)
+					e2e.Privileged(cleanup)(t)
 				}
 			})
 
@@ -550,7 +550,7 @@ from: %s
 				t, c.env.TestDir, fmt.Sprintf("issue1812-sandbox-%s-", testName), "")
 			t.Cleanup(func() {
 				if !t.Failed() {
-					e2e.Privileged(cleanup)
+					e2e.Privileged(cleanup)(t)
 				}
 			})
 
@@ -643,6 +643,25 @@ func (c *imgBuildTests) issue2347(t *testing.T) {
 	)
 }
 
+// When a symlink at /foo/bar targets ../bar then it should copy correctly when
+// /foo is copied between stages.
+func (c *imgBuildTests) issue2607(t *testing.T) {
+	image := filepath.Join(c.env.TestDir, "issue_2607.sif")
+
+	c.env.RunApptainer(
+		t,
+		e2e.WithProfile(e2e.RootProfile),
+		e2e.WithCommand("build"),
+		e2e.WithArgs(image, "testdata/regressions/issue_2607.def"),
+		e2e.PostRun(func(_ *testing.T) {
+			os.Remove(image)
+		}),
+		e2e.ExpectExit(
+			0,
+		),
+	)
+}
+
 // Check that the build process from an image doesn't fail when the source image
 // includes symlinks. Also confirm that the base environment files from the
 // source image are not overwritten (#3353).
