@@ -2,7 +2,7 @@
 //   Apptainer a Series of LF Projects LLC.
 //   For website terms of use, trademark policy, privacy policy and other
 //   project policies see https://lfprojects.org/policies
-// Copyright (c) 2019-2023, Sylabs Inc. All rights reserved.
+// Copyright (c) 2019-2025, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -37,7 +37,6 @@ import (
 	"github.com/apptainer/apptainer/internal/pkg/util/fs"
 	"github.com/apptainer/apptainer/internal/pkg/util/fs/squashfs"
 	"github.com/apptainer/apptainer/internal/pkg/util/gpu"
-	"github.com/apptainer/apptainer/internal/pkg/util/shell/interpreter"
 	"github.com/apptainer/apptainer/internal/pkg/util/starter"
 	"github.com/apptainer/apptainer/internal/pkg/util/user"
 	"github.com/apptainer/apptainer/pkg/build/types"
@@ -54,7 +53,6 @@ import (
 	"github.com/apptainer/apptainer/pkg/util/fs/proc"
 	"github.com/apptainer/apptainer/pkg/util/namespaces"
 	"github.com/apptainer/apptainer/pkg/util/rlimit"
-	lccgroups "github.com/opencontainers/runc/libcontainer/cgroups"
 	"github.com/opencontainers/runtime-spec/specs-go"
 	"golang.org/x/sys/unix"
 )
@@ -428,7 +426,13 @@ func (l *Launcher) Exec(ctx context.Context, image string, args []string, instan
 	if l.engineConfig.GetInstance() && !l.cfg.ShareNSMode {
 		err = l.starterInstance(loadOverlay, insideUserNs, instanceName, useSuid, cfg)
 	} else {
-		err = l.starterInteractive(loadOverlay, useSuid, cfg)
+		var imageFilename string
+		var fileInfoErr error
+		info, fileInfoErr := os.Stat(image)
+		if fileInfoErr == nil {
+			imageFilename = info.Name()
+		}
+		err = l.starterInteractive(loadOverlay, useSuid, cfg, imageFilename)
 	}
 
 	// Execution is finished.
@@ -887,11 +891,11 @@ func (l *Launcher) setNVLegacyConfig() error {
 	if err != nil {
 		sylog.Warningf("While finding nv ipcs: %v", err)
 	}
-	libs, bins, err := gpu.NvidiaPaths(gpuConfFile)
+	libs, bins, files, err := gpu.NvidiaPaths(gpuConfFile)
 	if err != nil {
 		sylog.Warningf("While finding nv bind points: %v", err)
 	}
-	l.addGPUBinds(libs, bins, ipcs, "nv")
+	l.addGPUBinds(libs, bins, ipcs, files, "nv")
 	return nil
 }
 
@@ -904,15 +908,18 @@ func (l *Launcher) setRocmConfig() error {
 	if err != nil {
 		sylog.Warningf("While finding ROCm bind points: %v", err)
 	}
-	l.addGPUBinds(libs, bins, []string{}, "rocm")
+	l.addGPUBinds(libs, bins, []string{}, []string{}, "rocm")
 	return nil
 }
 
 // addGPUBinds adds EngineConfig entries to bind the provided list of libs, bins, ipc files.
-func (l *Launcher) addGPUBinds(libs, bins, ipcs []string, gpuPlatform string) {
-	files := make([]string, len(bins)+len(ipcs))
+func (l *Launcher) addGPUBinds(libs, bins, ipcs, regularFiles []string, gpuPlatform string) {
+	files := make([]string, len(bins)+len(ipcs)+len(regularFiles))
 	if len(files) == 0 {
-		sylog.Warningf("Could not find any %s files on this host!", gpuPlatform)
+		if bins != nil {
+			// only warn if we requested files to bind
+			sylog.Warningf("Could not find any %s files on this host!", gpuPlatform)
+		}
 	} else {
 		if l.cfg.Writable {
 			sylog.Warningf("%s files may not be bound with --writable", gpuPlatform)
@@ -924,10 +931,16 @@ func (l *Launcher) addGPUBinds(libs, bins, ipcs []string, gpuPlatform string) {
 		for i, ipc := range ipcs {
 			files[i+len(bins)] = ipc
 		}
+		for i, file := range regularFiles {
+			files[i+len(bins)+len(ipcs)] = file
+		}
 		l.engineConfig.AppendFilesPath(files...)
 	}
 	if len(libs) == 0 {
-		sylog.Warningf("Could not find any %s libraries on this host!", gpuPlatform)
+		if libs != nil {
+			// only warn if we requested libraries to bind
+			sylog.Warningf("Could not find any %s libraries on this host!", gpuPlatform)
+		}
 	} else {
 		l.engineConfig.AppendLibrariesPath(libs...)
 	}
@@ -948,6 +961,12 @@ func (l *Launcher) setNamespaces() {
 			l.cfg.Network = "bridge"
 			l.engineConfig.SetNetwork(l.cfg.Network)
 		}
+		if l.cfg.NetnsPath != "" {
+			sylog.Warningf("cannot join existing --netns-path and create a new network namespace with --net/-n")
+		}
+		// unprivileged installation could not use fakeroot
+		// network because it requires a setuid installation
+		// so we fallback to none
 		if l.cfg.Fakeroot && l.cfg.Network != "none" {
 			// unprivileged installation could not use fakeroot
 			// network because it requires a setuid installation
@@ -962,6 +981,11 @@ func (l *Launcher) setNamespaces() {
 		}
 		l.generator.AddOrReplaceLinuxNamespace("network", "")
 	}
+	if l.cfg.NetnsPath != "" {
+		// Note - runtime code checks whether netns-path is permitted (root or
+		// allowed via apptainer.conf).
+		l.generator.AddOrReplaceLinuxNamespace("network", l.cfg.NetnsPath)
+	}
 	if l.cfg.Namespaces.UTS {
 		l.generator.AddOrReplaceLinuxNamespace("uts", "")
 	}
@@ -983,45 +1007,36 @@ func (l *Launcher) setNamespaces() {
 
 // setEnvVars sets the environment for the container, from the host environment, glads, env-file.
 func (l *Launcher) setEnvVars(ctx context.Context, args []string) error {
-	if l.cfg.EnvFile != "" {
+	if len(l.cfg.EnvFiles) > 0 {
 		currentEnv := append(
 			os.Environ(),
 			"APPTAINER_IMAGE="+l.engineConfig.GetImage(),
 		)
 
-		content, err := os.ReadFile(l.cfg.EnvFile)
-		if err != nil {
-			return fmt.Errorf("could not read %q environment file: %w", l.cfg.EnvFile, err)
+		// Read all environment files and put the variables into envFilesMap,
+		// environment variables in later files will take precedence.
+		envFilesMap := map[string]string{}
+		for _, envFile := range l.cfg.EnvFiles {
+			tempEnvMap, err := env.FileMap(ctx, envFile, args, currentEnv)
+			if err != nil {
+				return fmt.Errorf("while processing %s: %w", envFile, err)
+			}
+			sylog.Debugf("Setting environment variables from file %s", envFile)
+			envFilesMap = env.MergeMap(envFilesMap, tempEnvMap)
 		}
 
-		shellEnv, err := interpreter.EvaluateEnv(ctx, content, args, currentEnv)
-		if err != nil {
-			return fmt.Errorf("while processing %s: %w", l.cfg.EnvFile, err)
-		}
-		// --env variables will take precedence over variables
-		// defined by the environment file
-		sylog.Debugf("Setting environment variables from file %s", l.cfg.EnvFile)
-
+		// --env variables will take precedence over variables defined by the environment files
 		// Update Env with those from file
-		for _, envar := range shellEnv {
-			e := strings.SplitN(envar, "=", 2)
-			if len(e) != 2 {
-				sylog.Warningf("Ignore environment variable %q: '=' is missing", envar)
-				continue
-			}
-			// Don't attempt to overwrite bash builtin readonly vars
-			// https://github.com/sylabs/singularity/issues/1263
-			if _, ok := env.ReadOnlyVars[e[0]]; ok {
-				continue
-			}
+		for k, v := range envFilesMap {
 			// Ensure we don't overwrite --env variables with environment file
-			if _, ok := l.cfg.Env[e[0]]; ok {
-				sylog.Warningf("Ignore environment variable %s from %s: override from --env", e[0], l.cfg.EnvFile)
+			if _, ok := l.cfg.Env[k]; ok {
+				sylog.Warningf("Ignored environment file variable %s: override from --env", k)
 			} else {
-				l.cfg.Env[e[0]] = e[1]
+				l.cfg.Env[k] = v
 			}
 		}
 	}
+
 	// process --env and --env-file variables for injection
 	// into the environment by prefixing them with APPTAINERENV_
 	for envName, envValue := range l.cfg.Env {
@@ -1082,46 +1097,32 @@ func (l *Launcher) setCgroups(instanceName string) error {
 		return nil
 	}
 
-	hidePid := hidepidProc()
 	// If we are an instance, always use a cgroup if possible, to enable stats.
-	// root can always create a cgroup.
-	sylog.Debugf("During setting cgroups configuration, uid: %d, namespace.user: %t, fakeroot: %t, unprivileged: %t", l.uid, l.cfg.Namespaces.User, l.cfg.Fakeroot, namespaces.IsUnprivileged())
-	useCG := l.uid == 0 && !namespaces.IsUnprivileged()
-	// non-root needs cgroups v2 unified mode + systemd as cgroups manager.
-	if !useCG && lccgroups.IsCgroup2UnifiedMode() && l.engineConfig.File.SystemdCgroups && !namespaces.IsUnprivileged() && !l.cfg.Fakeroot && !hidePid {
-		if os.Getenv("XDG_RUNTIME_DIR") == "" || os.Getenv("DBUS_SESSION_BUS_ADDRESS") == "" {
-			sylog.Infof("Instance stats will not be available because XDG_RUNTIME_DIR")
-			sylog.Infof("  or DBUS_SESSION_BUS_ADDRESS is not set")
+	err := cgroups.CanUseCgroups(l.engineConfig.File.SystemdCgroups)
+	if err == nil {
+		// CanUseCgroups catches cases where fakeroot is already
+		// in effect but sometimes when l.cfg.Fakeroot is set it
+		// takes effect later, so also check that here
+		if l.cfg.Fakeroot {
+			err = fmt.Errorf("doesn't work with fakeroot mode")
+		} else if hidepidProc() {
+			err = fmt.Errorf("hidepid option is set on /proc mount")
+		} else {
+			sylog.Debugf("Using cgroup manager during setting cgroups configuration")
+			cg := cgroups.Config{}
+			cgJSON, err := cg.MarshalJSON()
+			if err != nil {
+				return err
+			}
+			l.engineConfig.SetCgroupsJSON(cgJSON)
 			return nil
 		}
-		useCG = true
-	}
-
-	if useCG {
-		sylog.Debugf("Using cgroup manager during setting cgroups configuration")
-		cg := cgroups.Config{}
-		cgJSON, err := cg.MarshalJSON()
-		if err != nil {
-			return err
-		}
-		l.engineConfig.SetCgroupsJSON(cgJSON)
-		return nil
-	}
-
-	if l.cfg.Fakeroot {
-		sylog.Debugf("Instance stats will not be available because of fakeroot mode")
-		return nil
-	}
-
-	if hidePid {
-		sylog.Debugf("Instance stats will not be available because of hidepid option is set on /proc mount")
-		return nil
 	}
 
 	if l.cfg.ShareNSMode {
-		sylog.Debugf("Instance stats will not be available - requires cgroups v2 with systemd as manager.")
+		sylog.Debugf("Instance stats will not be available - %v", err)
 	} else {
-		sylog.Infof("Instance stats will not be available - requires cgroups v2 with systemd as manager.")
+		sylog.Infof("Instance stats will not be available - %v", err)
 	}
 	return nil
 }
@@ -1129,7 +1130,7 @@ func (l *Launcher) setCgroups(instanceName string) error {
 // PrepareImage performs any image preparation required before execution.
 // This is currently limited to extraction or FUSE mount when using the user namespace,
 // and activating any image driver plugins that might handle the image mount.
-func (l *Launcher) prepareImage(c context.Context, insideUserNs bool, image string) error {
+func (l *Launcher) prepareImage(_ context.Context, insideUserNs bool, image string) error {
 	// initialize internal image drivers
 	var desiredFeatures imgutil.DriverFeature
 	if fs.IsFile(image) {
@@ -1197,9 +1198,9 @@ func (l *Launcher) prepareImage(c context.Context, insideUserNs bool, image stri
 }
 
 // starterInteractive executes the starter binary to run an image interactively, given the supplied engineConfig
-func (l *Launcher) starterInteractive(loadOverlay bool, useSuid bool, cfg *config.Common) error {
+func (l *Launcher) starterInteractive(loadOverlay bool, useSuid bool, cfg *config.Common, imageFilename string) error {
 	err := starter.Exec(
-		"Apptainer runtime parent",
+		"Apptainer runtime parent: "+imageFilename,
 		cfg,
 		starter.UseSuid(useSuid),
 		starter.LoadOverlayModule(loadOverlay),
@@ -1250,7 +1251,7 @@ func (l *Launcher) starterInstance(loadOverlay bool, insideUserNs bool, name str
 		if end-start > 0 {
 			output := make([]byte, end-start)
 			stderr.ReadAt(output, start)
-			fmt.Println(string(output))
+			fmt.Fprintln(os.Stderr, string(output))
 		}
 	}
 
@@ -1288,9 +1289,9 @@ func withPrivilege(uid uint32, cond bool, desc string, fn func() error) error {
 	return fn()
 }
 
-// hidepidProc checks if hidepid is set on /proc mount point, when this
-// option is an instance started with setuid workflow could not even be
-// joined later or stopped correctly.
+// hidepidProc checks if hidepid is set on /proc mount point.  When this
+// is set an instance started with setuid workflow could not even be
+// joined later or stopped correctly.  It also interferes with cgroups.
 func hidepidProc() bool {
 	entries, err := proc.GetMountInfoEntry("/proc/self/mountinfo")
 	if err != nil {
